@@ -50,6 +50,10 @@ class Layout(NodeFunc):
         super().__init__(None)
         self.op = op
 
+    @property
+    def graphviz_name(self):
+        return self.wrapped_name('layout')
+
     def __call__(self):
         num_layouts = self.op.data_formats.num_layouts()
         for i, layout in enumerate(range(num_layouts)):
@@ -66,6 +70,11 @@ class RankRange(ReportNodeFunc):
         self.obs_shapes_cons = [] # constraint based on shapes
         self.obs_args_cons = []
 
+    @property
+    def graphviz_name(self):
+        ind_name = self.op.index[self.sub_name].display_name(True)
+        return self.wrapped_name(ind_name)
+
     def add_schema_constraint(self, cons):
         # these functions are called with **index_ranks
         self.schema_cons.append(cons)
@@ -84,7 +93,13 @@ class RankRange(ReportNodeFunc):
         # these functions are called with obs_args, **index_ranks
         self.obs_args_cons.append(cons)
 
-    def __call__(self, obs_shapes, obs_args, **index_ranks):
+    def __call__(self, **kwargs):
+        if 'shapes' in kwargs:
+            obs_shapes = kwargs.pop('shapes')
+        if 'args' in kwargs:
+            obs_args = kwargs.pop('args')
+        index_ranks = kwargs
+        
         # Get the initial bounds consistent with the schema
         sch_lo, sch_hi = 0, 100000
         for cons in self.schema_cons:
@@ -109,9 +124,15 @@ class RankEquiv(NodeFunc):
     """
     Produce a range identical to the primary index
     """
-    def __init__(self, name):
+    def __init__(self, op, name):
+        self.op = op
         super().__init__(name)
 
+    @property
+    def graphviz_name(self):
+        ind_name = self.op.index[self.sub_name].display_name(True)
+        return self.wrapped_name(ind_name)
+
     def __call__(self, rank):
         yield rank
 
