@@ -17,18 +17,24 @@ def init_schema(op):
     op.arg_shape_int('rate', 'r')
     op.arg_unchecked('name')
 
-    op.gen_dims('b', 50)
-    op.gen_dims('k', 30)
-    op.gen_dims('f', 100)
+    op.gen_dims('b', 1, 50, 50, True)
+    op.gen_dims('k', 1, 30, 30, True)
+    op.gen_dims('f', 1, 100, 100, True)
     op.gen_dims_func('i', genlib.below_above, 'f', 300, False)  
-    op.gen_dims('l', 30)
-    op.gen_dims('r', 30)
+    op.gen_dims('l', 1, 30, 30, True)
+    op.gen_dims('r', 1, 30, 30, True)
 
     op.valid_dtypes('value', ('int32', 'float',))
     op.equate_dtypes('filters', 'value')
 
     op.comp_dims_cw('g', dilate, dilate_t, 'fr')
     op.comp_dims_cw('o', conv, conv_t, 'ig', 'padding')
+
+    op.dims_pred_rng('f', 1, None)
+    op.dims_pred_rng('k', 1, None)
+    op.dims_pred_rng('r', 1, None)
+    op.dims_pred_rng('l', 1, None)
+
     op.return_tensor('bol')
 
 
