@@ -14,18 +14,18 @@ def init_schema(op):
 
     op.arg_tensor('value', 'bik')
     op.arg_tensor('filters', 'flk')
-    op.arg_shape_tensor('output_shape', 'bol')
+    op.arg_shape_tensor('output_shape', 1, None, 'bol')
     op.arg_shape_int('rate', 'r')
     op.arg_option('padding', ('VALID', 'SAME'))
     op.arg_unchecked('name')
     op.return_tensor('bql')
 
-    op.gen_dims('b', 100)
-    op.gen_dims('f', 100)
+    op.gen_dims('b', 1, 100, 100, True)
+    op.gen_dims('f', 1, 100, 100, True)
     op.gen_dims_func('i', genlib.below_above, 'f', 1000, False)  
-    op.gen_dims('k', 30)
-    op.gen_dims('l', 30)
-    op.gen_dims('r', 30)
+    op.gen_dims('k', 1, 30, 30, True)
+    op.gen_dims('l', 1, 30, 30, True)
+    op.gen_dims('r', 1, 30, 30, True)
 
     op.comp_dims_cw('g', dilate, dilate_t, 'fr') 
 
