@@ -4,10 +4,10 @@ def init_schema(op):
     op.add_index('c', 'write address component', 1)
     op.add_index('w', 'write address', (1, 5))
 
-    op.gen_dims('r', 100)
-    op.gen_dims('e', 100)
-    op.gen_dims('c', 8)
-    op.gen_dims('w', 100)
+    op.gen_dims('r', 1, 100, 100, True)
+    op.gen_dims('e', 1, 100, 100, True)
+    op.gen_dims('c', 1, 8, 8, True)
+    op.gen_dims('w', 1, 100, 100, True)
 
     # op.limit_ranks('rc', 1, 10)
     # op.limit_ranks('re', 1, 10)
