@@ -6,6 +6,7 @@ import sys, io, os
 import re
 import itertools
 from random import Random
+from . import genlib
 from . import predicates as pr
 from . import generators as ge
 from . import infer as nf
@@ -17,6 +18,8 @@ from .redirect import stderr_redirector
 from .error import *
 from .fgraph import PredNode as P, GenNode as G, FuncNode as F
 from .base import ShapeKind
+from .procfuncs import proc_wrap
+
 
 """
 Every API call will mutate the Generative Graph and the Predicate Graph
@@ -157,9 +160,6 @@ class OpSchema(object):
         self.arguments = {}
         self.returns = [] 
 
-    def _set_gen_error_quotas(self, dtype_quota):
-        self.dtype_err_quota = dtype_quota
-
     def _pred_node(self, pred_class, name=None):
         name = fgraph.node_name(pred_class, name)
         return self.pred_graph.get(name, None)
@@ -207,6 +207,7 @@ class OpSchema(object):
             except BaseException as ex:
                 raise OpSchemaInternalError(ex)
             try:
+                # exit_code, ret_val = proc_wrap(self.framework_op, **self.arguments)
                 ret_val = self.framework_op(**self.arguments)
                 self._check_return(ret_val)
                 return ret_val
@@ -823,25 +824,22 @@ class OpSchema(object):
     def _prep_gen_inventory(self):
         self.avail_test_edits = 0
 
-    def _prep_gen_tests(self):
-        self.avail_test_edits = 1
-
     def generate_args(self, rand_seed=12345):
-        self._prep_gen_tests()
         live = self.gen_graph.values()
         out = [self._gen_node(ge.Args)]
         self.gen_rng.seed(rand_seed)
         for op_args in fgraph.gen_graph_values(live, out, self):
             yield op_args[0] # extract tuple element
 
-    def validate(self, out_dir, test_ids, skip_ids, dtype_err_quota, rand_seed,
-            show_traceback=True):
+    def validate(self, out_dir, test_ids, skip_ids, dtype_err_quota,
+            test_edits, rand_seed, show_traceback=True):
         if not os.path.exists(out_dir):
             raise RuntimeError(
                 f'{type(self).__qualname__}: Could not open output path '
                 f'\'{out_dir}\' for report generation')
 
-        self._set_gen_error_quotas(dtype_err_quota)
+        self.dtype_err_quota = dtype_err_quota
+        self.avail_test_edits = test_edits
 
         report_fh = open(os.path.join(out_dir, f'{self.op_path}.txt'), 'w')
         summary_fh = open(os.path.join(out_dir, f'{self.op_path}.sum.txt'), 'w')
@@ -868,13 +866,16 @@ class OpSchema(object):
             # continue
             string_err = io.BytesIO()
             try:
+                # proc_wrap(self, op_args)
                 with stderr_redirector(string_err):
                     self.wrapped_op(**arg_dict)
+                    # pass
             except (OpSchemaInternalError, SchemaError) as ex:
                 print(string_err.getvalue().decode('UTF-8'))
                 raise ex
             except BaseException as ex:
                 pass
+                # raise ex
 
             if self.op_error is None:
                 cat = 'TN' if self.framework_exc_msg is None else 'FN'
@@ -953,7 +954,7 @@ class OpSchema(object):
                 ranks_gnode.append_parent_sn(idx_gnode)
 
                 G.set_registry(self.inf_graph)
-                iobj = nf.RankEquiv(idx)
+                iobj = nf.RankEquiv(self, idx)
                 ipa = self.inf_graph[primary_idx]
                 idx_inode = G.add_node_sn(iobj, ipa) 
                 ranks_inode.append_parent_sn(idx_inode)
@@ -975,7 +976,7 @@ class OpSchema(object):
             G.set_registry(self.inf_graph)
             sigs_inode = self._inf_node(ge.SigMap)
             idx_iobj = nf.RankRange(self, idx)
-            idx_inode = G.add_node_sn(idx_iobj, self.obs_shapes, self.obs_args)
+            idx_inode = G.add_node_sn(idx_iobj)
             ranks_inode.append_parent_sn(idx_inode)
 
             if isinstance(rank_range, tuple):
@@ -993,15 +994,6 @@ class OpSchema(object):
             else:
                 pass
 
-            pri_inds = [ind.idx for ind in self.index.values() if ind.primary()]
-            for pidx in pri_inds:
-                if pidx == idx:
-                    continue
-                gpa = self.gen_graph[pidx]
-                idx_gnode.append_parent_sn(gpa)
-                ipa = self.inf_graph[pidx]
-                idx_inode.append_parent_sn(ipa)
-
         else:
             raise SchemaError(
                 f'{type(self).__qualname__}: Got constraint \'{rank_cons}\''
@@ -1141,30 +1133,38 @@ class OpSchema(object):
         `arg_names` must be names of arguments passed to the op
         arg_vals are the runtime resolved values of the argument names
         """
-        def gen(rng, *args):
-            yield from func(*args)
-
+        gen = genlib.GenFromFunc(func)
         return self.gen_dims_func(out_sig, gen, in_sig, int(1e10), False, *arg_names)
 
-    def gen_dims(self, out_idx, max_prod):
+    def gen_dims(self, out_idx, min_val, max_val, max_prod, gen_zero=False):
         """
-        Generate dimensions of {out_idx} of appropriate rank such that the
-        product is in [1, max_prod]
+        Generate dimensions of `out_idx` of appropriate rank such that each
+        dimension is in [min_val, max_val] and product is in [0, max_prod].
+        If `gen_zero`, generate zero as an additional output.
         """
-        def gen(rng):
-            yield 1, None
+        gen = genlib.GenRange(min_val, max_val, gen_zero)
         try:
             self.gen_dims_func(out_idx, gen, '', max_prod, False)
         except SchemaError as ex:
             raise SchemaError(f'gen_dims got error {ex}')
 
+    def gen_dims_prod(self, out_idx, max_prod):
+        """
+        Generate dimensions of {out_idx} of appropriate rank such that the
+        product is in [1, max_prod]
+        """
+        gen = genlib.GenRange(1, None)
+        try:
+            self.gen_dims_func(out_idx, gen, '', max_prod, False)
+        except SchemaError as ex:
+            raise SchemaError(f'gen_dims_prod got error {ex}')
+
     def gen_dims_rng(self, out_idx, min_val, max_val):
         """
         Generate dimensions of {out_idx} of appropriate rank such that the
         product is in [min_prod, max_prod]
         """
-        def gen(rng):
-            yield min_val, max_val
+        gen = genlib.GenRange(min_val, max_val)
         try:
             self.gen_dims_func(out_idx, gen, '', int(1e10), False)
         except SchemaError as ex:
@@ -1591,15 +1591,17 @@ class OpSchema(object):
         kind = ShapeKind.Int
         self._arg_shape_func(arg_name, (index,), arg_p, gen_obj, kind)
 
-    def arg_shape_tensor(self, arg_name, *sigs):
+    def arg_shape_tensor(self, arg_name, min_elem_val, max_elem_val, *sigs):
         """
         Register {arg_name} as a 1D integer tensor whose elements define the
         shape of a signature.  
+
+        Check that every element is in [`min_elem_val`, `max_elem_val`].
         """
         P.set_registry(self.pred_graph)
         schema = self._pred_node(pr.Schema)
         gen_obj = ge.ShapeTensor(arg_name)
-        pred_obj = pr.ShapeTensor(arg_name, gen_obj)
+        pred_obj = pr.ShapeTensor(arg_name, gen_obj, min_elem_val, max_elem_val)
         arg_p = P.add_node(pred_obj, schema)
         kind = ShapeKind.Tensor
         self._arg_shape_func(arg_name, sigs, arg_p, gen_obj, kind)
@@ -1707,6 +1709,7 @@ class OpSchema(object):
             pri_idx = self.index[idx].pri_idx
             inode = self.inf_graph[pri_idx]
             inode.func.add_args_constraint(cons)
+            inode.append_parent_sn(self.obs_args)
 
     def rank_dims_constraint(self, func, rank_sig, shape_arg):
         """
@@ -1718,6 +1721,7 @@ class OpSchema(object):
             pri_idx = self.index[idx].pri_idx
             inode = self.inf_graph[pri_idx]
             inode.func.add_shapes_constraint(cons)
+            inode.append_parent_sn(self.obs_shapes)
 
     def dims_pred(self, pred_name, pfunc, pfunc_t, indices):
         """
@@ -1822,15 +1826,15 @@ class OpSchema(object):
 
     def _dot_graph(self, nodes, out_file):
         import graphviz
-        dot = graphviz.Digraph(graph_attr={'rankdir': 'LR'})
-        names = { n.name: n.name.replace(':', '_') for n in nodes }
+        nodes = list(nodes)
+        dot = graphviz.Digraph(graph_attr={'rankdir': 'LR'}, format='svg')
+        names = { n.name: n.func.graphviz_name for n in nodes }
         for node in nodes:
             is_arg = (node in self.arg_gen_nodes.values())
             color = 'red' if is_arg else 'black'
-            dot.node(names[node.name], node.name, color=color)
+            dot.node(names[node.name], names[node.name], color=color)
             vtype = node.vararg_type
-            for i, (pa,sn) in enumerate(zip(node.parents,
-                node.use_parent_subname)):
+            for i, (pa,sn) in enumerate(zip(node.parents, node.use_parent_subname)):
                 if i < node.num_named_pars:
                     color = 'black'
                 elif vtype == fgraph.VarArgs.Positional:
@@ -1839,10 +1843,9 @@ class OpSchema(object):
                     color = 'purple' if sn else 'blue'
                 else:
                     color = 'black'
-                dot.edge(names[node.name], names[pa.name],
-                        _attributes={'color': color})
-        dot.render(out_file)
-        print(f'Wrote {out_file}.pdf')
+                dot.edge(names[node.name], names[pa.name], _attributes={'color': color})
+        dot.render(out_file, cleanup=True)
+        print(f'Wrote {out_file}.svg')
 
     def print_graphs(self, out_dir):
         nodes = self.pred_graph.values()
